#include <Python.h>

#include<unistd.h>
#include<sys/types.h>
#include<sys/mman.h>
#include<fcntl.h>

#include "peekpoke.h"
#include "ep93xx_adc.h"

static PyObject *ErrorObject;

#define DATA_PAGE    0x12C00000
#define NUM_CHANNELS 4
int stored_calibration[NUM_CHANNELS][2];
int calibrated = 0;

static int devmem; // /dev/mem fd
static unsigned long adc_page, syscon_page;
static char *dr_page;
static unsigned char *start;

static volatile unsigned int *PEDR, *PEDDR, *PBDR, *PBDDR, *GPIOBDB;

static PyObject *
ts7400_get(PyObject *self, PyObject *args)
{
    int bit;

    if (!PyArg_ParseTuple(args, "i", &bit))
        return NULL;
    return Py_BuildValue("i", *PEDR & (1 << bit));
}

static PyObject *
ts7400_put(PyObject *self, PyObject *args)
{
    int bit, val;

    if (!PyArg_ParseTuple(args, "ii", &bit, &val))
        return NULL;
    if (val) {
        *PEDR |= (1 << bit);
    } else {
        *PEDR &= ~(1 << bit);
    }
    Py_INCREF(Py_None);
    return Py_None;
}


void read_calibration(void) 
{
	int i,j,k;
	unsigned short cal[NUM_CHANNELS*2];
	// read 16 calibration bytes into buffer
	FILE *f = fopen("/etc/ADC-calibration.dat","r");
	if (!f) return;

	if (fread(cal,NUM_CHANNELS*4,1,f) == 1) 
	{
		k = 0;
		for(j=0;j<2;j++)
			for(i=0;i<NUM_CHANNELS;i++)
				stored_calibration[i][j] = cal[k++];
		calibrated++;
	}
	fclose(f);
}

double get_volts(int val, int zero, int range)
{
	if(val <= 0x7000)
		val = val + 0x10000;

	val = val - zero;

	return  ((double)val * 3.3) / range;
}

static int channels[NUM_CHANNELS] = {ADC_CH0,ADC_CH1,ADC_CH2,ADC_CH3};

static PyObject *
ts7400_analog(PyObject *self, PyObject *args)
{
    int channel;
    int sample;
    double volts;

    if (!PyArg_ParseTuple(args, "i", &channel))
        return NULL;

    sample = read_channel(adc_page, channels[channel]);
    volts = get_volts(sample, stored_calibration[channel][0],stored_calibration[channel][1]);
    volts = get_volts(sample, 0x9E58, 0xC350);
    return Py_BuildValue("(d, i)", volts, sample);
}

static PyObject *
ts7400_calibrated(PyObject *self, PyObject *args)
{
    return Py_BuildValue("i", calibrated);
}

static PyMethodDef ts7400_methods[] = {
    {"get",  (PyCFunction)ts7400_get, METH_VARARGS,
     "Get a single ts7400 pin's value"},
    {"put",  (PyCFunction)ts7400_put, METH_VARARGS,
     "Put a single ts7400 pin's value"},
    {"analog",  (PyCFunction)ts7400_analog, METH_VARARGS,
     "Read a single ts7400 pin's analog value"},
    {"calibrated",  (PyCFunction)ts7400_calibrated, METH_VARARGS,
     "True if the analog values are calibrated voltages or false if raw ADC counts"},
    {NULL, NULL, 0, NULL}        /* Sentinel */
};


void
initts7400(void)
{
	PyObject *m, *d;

	/* initialize the memory pointers */
        devmem = open("/dev/mem", O_RDWR);
        if (devmem < 0) PyErr_SetFromErrno(PyExc_OSError);
        start = mmap(0, getpagesize(),
		PROT_READ|PROT_WRITE, MAP_SHARED, devmem, 0x80840000);
        if (start == MAP_FAILED) PyErr_SetFromErrno(PyExc_OSError);
        PEDR = (unsigned int *)(start + 0x20); //port e data
        PEDDR = (unsigned int *)(start + 0x24); //port e direction
        *PEDDR = 0xff; //all output (just 2 bits)
	PBDR = (unsigned int *)(start + 0x04); //port b
	PBDDR = (unsigned int *)(start + 0x14); //port b direction
	GPIOBDB = (unsigned int *)(start + 0xC4); // debounce on port b
	*PBDDR = 0xf0; //upper nibble output, lower nibble input
	*GPIOBDB = 0x01; //enable debounce on bit 0

	dr_page = mmap(0, getpagesize(), PROT_READ|PROT_WRITE, 
		MAP_SHARED, devmem, DATA_PAGE);
        if (dr_page == MAP_FAILED) PyErr_SetFromErrno(PyExc_OSError);
	adc_page = (unsigned long)mmap(0, getpagesize(), PROT_READ|PROT_WRITE,
		MAP_SHARED, devmem, ADC_PAGE);
        if ((void *)adc_page == MAP_FAILED) PyErr_SetFromErrno(PyExc_OSError);
	syscon_page = (unsigned long)mmap(0, getpagesize(), PROT_READ|PROT_WRITE,
		MAP_SHARED, devmem, SYSCON_PAGE);
        if ((void *)syscon_page == MAP_FAILED) PyErr_SetFromErrno(PyExc_OSError);

	init_ADC(adc_page, syscon_page);
	read_calibration();

	/* Create the module and add the functions */
	m = Py_InitModule4("ts7400", ts7400_methods,
		"Digital I/O for the TS-7400",
		(PyObject*)NULL,PYTHON_API_VERSION);

	/* Add some symbolic constants to the module */
	d = PyModule_GetDict(m);
	ErrorObject = PyString_FromString("ts7400.error");
	PyDict_SetItemString(d, "error", ErrorObject);

	/* XXXX Add constants here */
	
	/* Check for errors */
	if (PyErr_Occurred())
		Py_FatalError("can't initialize module ts7400");
}


